/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.reading.time.service.impl;

import com.liferay.portal.kernel.model.GroupedModel;
import com.liferay.portal.kernel.service.ClassNameLocalService;
import com.liferay.portal.spring.extender.service.ServiceReference;
import com.liferay.reading.time.calculator.ReadingTimeCalculator;
import com.liferay.reading.time.model.ReadingTimeEntry;
import com.liferay.reading.time.service.base.ReadingTimeEntryLocalServiceBaseImpl;

import java.time.Duration;

import java.util.Optional;

/**
 * @author Alejandro Tardín
 */
public class ReadingTimeEntryLocalServiceImpl
	extends ReadingTimeEntryLocalServiceBaseImpl {

	@Override
	public ReadingTimeEntry addReadingTimeEntry(
		GroupedModel groupedModel, Duration readingTimeDuration) {

		return addReadingTimeEntry(
			groupedModel.getGroupId(),
			_classNameLocalService.getClassNameId(groupedModel.getModelClass()),
			(Long)groupedModel.getPrimaryKeyObj(), readingTimeDuration);
	}

	@Override
	public ReadingTimeEntry addReadingTimeEntry(
		long groupId, long classNameId, long classPK,
		Duration readingTimeDuration) {

		long entryId = counterLocalService.increment();

		ReadingTimeEntry entry = readingTimeEntryPersistence.create(entryId);

		entry.setGroupId(groupId);
		entry.setClassNameId(classNameId);
		entry.setClassPK(classPK);
		entry.setReadingTime(readingTimeDuration.toMillis());

		return readingTimeEntryPersistence.update(entry);
	}

	@Override
	public ReadingTimeEntry deleteReadingTimeEntry(GroupedModel groupedModel) {
		ReadingTimeEntry readingTimeEntry = fetchReadingTimeEntry(groupedModel);

		if (readingTimeEntry != null) {
			return deleteReadingTimeEntry(readingTimeEntry);
		}

		return null;
	}

	@Override
	public ReadingTimeEntry deleteReadingTimeEntry(
		long groupId, long classNameId, long classPK) {

		ReadingTimeEntry readingTimeEntry = fetchReadingTimeEntry(
			groupId, classNameId, classPK);

		if (readingTimeEntry != null) {
			return deleteReadingTimeEntry(readingTimeEntry);
		}

		return null;
	}

	@Override
	public ReadingTimeEntry fetchOrAddReadingTimeEntry(
		GroupedModel groupedModel) {

		ReadingTimeEntry readingTimeEntry = fetchReadingTimeEntry(groupedModel);

		if (readingTimeEntry != null) {
			return readingTimeEntry;
		}

		return updateReadingTimeEntry(groupedModel);
	}

	@Override
	public ReadingTimeEntry fetchReadingTimeEntry(GroupedModel groupedModel) {
		return fetchReadingTimeEntry(
			groupedModel.getGroupId(),
			_classNameLocalService.getClassNameId(groupedModel.getModelClass()),
			(Long)groupedModel.getPrimaryKeyObj());
	}

	@Override
	public ReadingTimeEntry fetchReadingTimeEntry(
		long groupId, long classNameId, long classPK) {

		return readingTimeEntryPersistence.fetchByG_C_C(
			groupId, classNameId, classPK);
	}

	@Override
	public ReadingTimeEntry updateReadingTimeEntry(GroupedModel groupedModel) {
		Optional<Duration> readingTimeOptional =
			_readingTimeCalculator.calculate(groupedModel);

		if (readingTimeOptional.isPresent()) {
			return updateReadingTimeEntry(
				groupedModel.getGroupId(),
				_classNameLocalService.getClassNameId(
					groupedModel.getModelClass()),
				(Long)groupedModel.getPrimaryKeyObj(),
				readingTimeOptional.get());
		}

		return null;
	}

	@Override
	public ReadingTimeEntry updateReadingTimeEntry(
		long groupId, long classNameId, long classPK,
		Duration readingTimeDuration) {

		ReadingTimeEntry readingTimeEntry = fetchReadingTimeEntry(
			groupId, classNameId, classPK);

		if (readingTimeEntry == null) {
			return addReadingTimeEntry(
				groupId, classNameId, classPK, readingTimeDuration);
		}
		else {
			readingTimeEntry.setReadingTime(readingTimeDuration.toMillis());

			return updateReadingTimeEntry(readingTimeEntry);
		}
	}

	@ServiceReference(type = ClassNameLocalService.class)
	private ClassNameLocalService _classNameLocalService;

	@ServiceReference(type = ReadingTimeCalculator.class)
	private ReadingTimeCalculator _readingTimeCalculator;

}